/*******************************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 ******************************************************************************/
package org.apache.olingo.odata2.api.annotation.edm;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * <p>Annotation for a EDM/CSDL EntityType element.</p>
 * <p>EdmEntityType holds a set of related information like EdmPrimitiveType, EdmComplexType
 * and EdmNavigation properties.</p>
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface EdmEntityType {
  /**
   * Define the name for the EDM EntityType.
   * If not set a default value has to be generated by the EDM provider.
   * 
   * @return name for the EDM EntityType
   */
  String name() default "";

  /**
   * Define the namespace for the EDM EntityType.
   * If not set a default value has to be generated by the EDM provider.
   * 
   * @return namespace for the EDM EntityType
   */
  String namespace() default "";
}